@@ -1,3 +1,4 @@
+from .primitives import *
 from .calc import *
 from .colls import *
 from .decorators import *
@@ -10,16 +11,4 @@ from .flow import *
 from .objects import *
 from .namespaces import namespace
 from .debug import *
-
-
-def inc(x):
-    return x + 1
-
-def dec(x):
-    return x - 1
-
-def even(x):
-    return x % 2 == 0
-
-def odd(x):
-    return x % 2 == 1
+from .primitives import *
