@@ -7,9 +7,11 @@ from .decorators import decorator
 
 __all__ = ['ignore', 'silent', 'retry', 'fallback',
            'limit_error_rate', 'ErrorRateExceeded',
-           'collecting', 'joining']
+           'post_processing', 'collecting', 'joining']
 
 
+### Error handling utilities
+
 # Not using @decorator here for speed,
 # since @ignore and @silent should be used for very simple and fast functions
 def ignore(errors, default=None):
@@ -83,9 +85,13 @@ def limit_error_rate(fails, timeout, exception=ErrorRateExceeded):
     return decorator
 
 
+### Post processing decorators
+
 @decorator
-def collecting(call):
-    return list(call())
+def post_processing(call, func):
+    return func(call())
+
+collecting = post_processing(list)
 
 @decorator
 def joining(call, sep):
