@@ -1,15 +1,16 @@
 from operator import add
 from itertools import islice, ifilter, imap, izip, chain, tee, ifilterfalse, dropwhile, takewhile, \
                       groupby
-from collections import defaultdict, Sequence
+from collections import defaultdict, deque, Sequence
 
+from .primitives import EMPTY
 from .funcs import partial
 from .funcmakers import wrap_mapper, wrap_selector
 
 
 __all__ = [
     'count', 'cycle', 'repeat', 'repeatedly', 'iterate',
-    'take', 'drop', 'first', 'second', 'rest', 'ilen',
+    'take', 'drop', 'first', 'second', 'last', 'rest', 'butlast', 'ilen',
     'map', 'filter', 'imap', 'ifilter', 'remove', 'iremove', 'keep', 'ikeep', 'without', 'iwithout',
     'concat', 'iconcat', 'chain', 'cat', 'icat', 'mapcat', 'imapcat',
     'izip', 'interleave', 'interpose', 'distinct',
@@ -21,9 +22,6 @@ __all__ = [
 ]
 
 
-EMPTY = object() # Used as default for optional arguments
-
-
 # Re-export
 from itertools import count, cycle, repeat
 
@@ -49,11 +47,40 @@ def first(seq):
 def second(seq):
     return first(rest(seq))
 
+def last(seq):
+    if hasattr(seq, '__getslice__'):
+        try:
+            return seq[-1]
+        except IndexError:
+            return None
+
+    item = None
+    for item in iter(seq):
+        pass
+    return item
+
 def rest(seq):
     return drop(1, seq)
 
+def butlast(seq):
+    it = iter(seq)
+    try:
+        prev = next(it)
+    except StopIteration:
+        pass
+    else:
+        for item in it:
+            yield prev
+            prev = item
+
 def ilen(seq):
-    return sum(1 for _ in seq)
+    """
+    Consume an iterable not reading it into memory; return the number of items.
+    Implementation borrowed from http://stackoverflow.com/a/15112059/753382
+    """
+    counter = count()
+    deque(izip(seq, counter), maxlen=0)  # (consume at C speed)
+    return next(counter)
 
 
 # TODO: tree-seq equivalent
@@ -142,12 +169,14 @@ def split_by(pred, seq):
 # NOTE: should I name it cluster? to distinguish from itertools.groupby
 #       or just group?
 # NOTE: should it return OrderedDict to preserve order of keys not just values?
+@wrap_mapper
 def group_by(f, seq):
     result = defaultdict(list)
     for item in seq:
         result[f(item)].append(item)
     return result
 
+@wrap_mapper
 def count_by(f, seq):
     result = defaultdict(int)
     for item in seq:
@@ -193,6 +222,7 @@ def ichunks(n, step, seq=EMPTY):
 def chunks(n, step, seq=EMPTY):
     return list(ichunks(n, step, seq))
 
+@wrap_selector
 def ipartition_by(f, seq):
     for g, items in groupby(seq, f):
         yield items
@@ -201,9 +231,9 @@ def partition_by(f, seq):
     return map(list, ipartition_by(f, seq))
 
 
-def with_prev(seq):
+def with_prev(seq, fill=None):
     a, b = tee(seq)
-    return izip(a, chain([None], b))
+    return izip(a, chain([fill], b))
 
 
 def ireductions(f, seq, acc=EMPTY):
